/*
Created by: X
http://www.createdbyx.com/

xMessages is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
#endregion

namespace xMessages
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class MessageComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        protected internal Queue<MessageInformation> messages;
        private SpriteBatch sb;
        private int topMargin = 4;
        private int bottomMargin = 4;
        private int leftMargin = 4;
        private int rightMargin = 4;
        private int imageTextMargin = 4;
        private MessageInformation currentInfo;
        private IMessageControl control;
        private string contentControlName;
        private IMessageControl contentControl;
        private Rectangle textRect;
        private Rectangle imageRect;

        /// <summary>
        /// Gets/Sets the name of the control that will contain the image and text.
        /// </summary>
        public string ContentControlName
        {
            get { return contentControlName; }
            set
            {
                contentControlName = value;
                if (this.control != null)
                {
                    if (string.Compare(this.control.Name, this.contentControlName, true) == 0)
                    {
                        this.contentControl = this.control;
                    }
                    else
                    {
                        IMessageControl p = this.control.Children.FindControl<IMessageControl>(this.contentControlName, true, true);
                        if (p != null)
                        {
                            this.contentControl = p;
                        }
                    }

                }
            }
        }

        /// <summary>
        /// Gets/Sets the control that will be used as the skin for the dialog.
        /// </summary>
        public IMessageControl Control
        {
            get { return control; }
            set { control = value; }
        }

        /// <summary>
        /// Gets/Sets the position that the control will be located on screen.
        /// </summary>
        public int Left
        {
            get
            {
                if (this.control != null)
                {
                    return this.control.Left;
                }
                else
                {
                    return 0;
                }
            }
            set
            {
                if (this.control != null)
                {
                    this.control.Left = value;
                }
            }
        }

        /// <summary>
        /// Gets/Sets the position that the control will be located on screen.
        /// </summary>
        public int Top
        {
            get
            {
                if (this.control != null)
                {
                    return this.control.Top;
                }
                else
                {
                    return 0;
                }
            }
            set
            {
                if (this.control != null)
                {
                    this.control.Top = value;
                }
            }
        }

        /// <summary>
        /// Gets/Sets the width of the control that will be displayed on screen.
        /// </summary>
        public int Width
        {
            get
            {
                if (this.control != null)
                {
                    return this.control.Width;
                }
                else
                {
                    return 0;
                }
            }
            set
            {
                if (this.control != null)
                {
                    this.control.Width = value;
                }
            }
        }

        /// <summary>
        /// Gets/Sets the height of the control that will be displayed on screen.
        /// </summary>
        public int Height
        {
            get
            {
                if (this.control != null)
                {
                    return this.control.Height;
                }
                else
                {
                    return 0;
                }
            }
            set
            {
                if (this.control != null)
                {
                    this.control.Height = value;
                }
            }
        }

        /// <summary>
        /// Gets/Sets the distance in pixels between the text and the text.
        /// </summary>
        public int ImageTextMargin
        {
            get { return imageTextMargin; }
            set { imageTextMargin = value; }
        }

        /// <summary>
        /// Sets the value of all the margins except for the ImageTextMargin.
        /// </summary>
        /// <param name="value">Left, Top, Right, Bottom margins will be set to this value.</param>
        public void SetMargins(int value)
        {
            this.leftMargin = value;
            this.rightMargin = value;
            this.topMargin = value;
            this.bottomMargin = value;
        }

        /// <summary>
        /// Sets the value of the outside margins.
        /// </summary>
        /// <param name="left">Sets the LeftMargin property.</param>
        /// <param name="top">Sets the TopMargin property.</param>
        /// <param name="right">Sets the RightMargin property.</param>
        /// <param name="bottom">Sets the BottomMargin property.</param>
        public void SetMargins(int left, int top, int right, int bottom)
        {
            this.leftMargin = left;
            this.rightMargin = right;
            this.topMargin = top;
            this.bottomMargin = bottom;
        }

        /// <summary>
        /// Sets the value of all the margins including the ImageTextMargin.
        /// </summary>
        /// <param name="value">Left, Top, Right, Bottom margins will be set to this value.</param>
        /// <param name="imagetext">Sets the value of the ImageTextMargin property.</param>
        public void SetMargins(int value, int imagetext)
        {
            this.SetMargins(value);
            this.imageTextMargin = imagetext;
        }

        /// <summary>
        /// Sets the value of all the margins including the ImageTextMargin.
        /// </summary>
        /// <param name="left">Sets the LeftMargin property.</param>
        /// <param name="top">Sets the TopMargin property.</param>
        /// <param name="right">Sets the RightMargin property.</param>
        /// <param name="bottom">Sets the BottomMargin property.</param>
        /// <param name="imagetext">Sets the value of the ImageTextMargin property.</param>
        public void SetMargins(int left, int top, int right, int bottom, int imagetext)
        {
            this.SetMargins(left, top, right, bottom);
            this.imageTextMargin = imagetext;
        }

        /// <summary>
        /// Gets/Sets the value for the right margin.
        /// </summary>
        public int RightMargin
        {
            get { return rightMargin; }
            set { rightMargin = value; }
        }

        /// <summary>
        /// Gets/Sets the value for the left margin.
        /// </summary>
        public int LeftMargin
        {
            get { return leftMargin; }
            set { leftMargin = value; }
        }

        /// <summary>
        /// Gets/Sets the value for the bottom margin.
        /// </summary>
        public int BottomMargin
        {
            get { return bottomMargin; }
            set { bottomMargin = value; }
        }

        /// <summary>
        /// Gets/Sets the value for the top margin.
        /// </summary>
        public int TopMargin
        {
            get { return topMargin; }
            set { topMargin = value; }
        }

        /// <summary>
        /// Gets/Sets the position in screen coordanants that the message window will be displayed at.
        /// </summary>
        public Rectangle DestinationRectangle
        {
            get
            {
                if (this.control != null)
                {
                    return this.control.Bounds;
                }
                else
                {
                    return Rectangle.Empty;
                }
            }
            set
            {
                if (this.control != null)
                {
                    this.control.Bounds = value;
                }
            }
        }

        /// <summary>
        /// Adds a message to the queue.
        /// </summary>
        /// <param name="info"></param>
        public void QueueMessage(MessageInformation info)
        {
            if (info == null) throw new ArgumentNullException("info");
            this.messages.Enqueue(info);
        }

        /// <summary>
        /// Cleas all messags from the queue.
        /// </summary>
        /// <remarks>If there is a currently displayed message Clear will end the message before clearing oput all other messages in stored in the queue.</remarks>
        public void Clear()
        {
            if (this.currentInfo != null)
            {
                this.currentInfo.DoEnded();
                if (this.currentInfo.DisposeOnEnded)
                {
                    this.currentInfo.Dispose();
                }
                this.currentInfo = null;
            }
            this.messages.Clear();
        }

        /// <summary>
        /// Removes the next message from the queue.
        /// </summary>
        public MessageInformation DequeueMessage()
        {
            if (this.currentInfo != null)
            {
                this.currentInfo.DoEnded();
                if (this.currentInfo.DisposeOnEnded)
                {
                    this.currentInfo.Dispose();
                }
                MessageInformation Tmp = this.currentInfo;
                this.currentInfo = null;
                return Tmp;
            }

            return null;
        }

        /// <summary>
        /// Retrieves the next message from the queue without removing it from the queue.
        /// </summary>
        /// <returns></returns>
        public MessageInformation PeekMessage()
        {
            if (this.messages.Count > 0)
            {
                return this.messages.Peek();
            }
            else
            {
                return null;
            };
        }

        /// <summary>
        /// Public constructor
        /// </summary>
        /// <param name="game">A refreence to a game object.</param>
        public MessageComponent(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
            this.messages = new Queue<MessageInformation>();
        }


        protected override void Dispose(bool disposing)
        {
            this.control = null;
            this.sb.Dispose();
            this.sb = null;
            // this.skintexture = null;
            this.messages.Clear();
            this.messages = null;
            base.Dispose(disposing);
        }

        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            /// create the sprite batch is not already created.
            if (this.sb == null) { this.sb = new SpriteBatch(this.GraphicsDevice); }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // Check if a message is being displayed and has timed out
            if (this.currentInfo != null && this.currentInfo.DisplayTime != TimeSpan.Zero && (gameTime.TotalGameTime > this.currentInfo.startTime + this.currentInfo.DisplayTime))
            {
                this.currentInfo.DoEnded();
                if (this.currentInfo.DisposeOnEnded)
                {
                    this.currentInfo.Dispose();
                }
                this.currentInfo = null;
            }

            // check if there are mssages in the queue and weather or not there is no currently displayed message
            if (this.messages.Count > 0 && this.currentInfo == null)
            {
                // get the next item from the queue
                this.currentInfo = this.messages.Dequeue();
                this.currentInfo.SetStartTime(gameTime.TotalGameTime);
                if(this.currentInfo.TextEffect != null) this.currentInfo.TextEffect.Reset();
                this.currentInfo.DoStarted();
            }

           this.textRect = CalcTextRectangle(this.contentControl);
            this.imageRect = CalcImageRectangle(this.contentControl);

            // update the current message
            if (this.currentInfo != null) this.currentInfo.Update(gameTime, this.textRect, this.imageRect);

            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // Ceck if there is a message that is being displayed
            if (this.currentInfo != null)
            {
                Rectangle sr = this.GraphicsDevice.ScissorRectangle;
                bool se = this.GraphicsDevice.RenderState.ScissorTestEnable;
                this.GraphicsDevice.ScissorRectangle = this.DestinationRectangle;
                this.GraphicsDevice.RenderState.ScissorTestEnable = true;

                // begin drawing message
                this.sb.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);

                // draw the control
                if (this.control != null)
                {
                    this.control.Draw(gameTime, this.sb);
                }

                //NOTE: Not sure why I need to call end and restart the drawing again. If I don't it will not draw properly.
                //      most  likly has something to do with how the soritebatch deals with the SissorRectangle
                this.sb.End();
                this.sb.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);

                // set the sissor rectangle to the bounds of the content control
                this.GraphicsDevice.ScissorRectangle = this.contentControl.ScreenBounds;

                IMessageControl ctrl = null;
                if (this.control != null) { ctrl = this.control; }
                if (this.contentControl != null) { ctrl = this.contentControl; }
                if (ctrl != null)
                {
                    if (this.currentInfo.ImageEffect != null)
                    {
                        this.currentInfo.ImageEffect.Draw(gameTime, this.currentInfo, this.imageRect, this.sb);
                    }
                    // use the text effect to draw the text for the message
                    if (this.currentInfo.TextEffect != null)
                    {
                        this.currentInfo.TextEffect.Draw(gameTime, this.currentInfo, this.textRect, this.sb);
                    }
                }

                this.sb.End();

                this.GraphicsDevice.RenderState.ScissorTestEnable = se;
                this.GraphicsDevice.ScissorRectangle = sr;
            }

            base.Draw(gameTime);
        }

        private Rectangle CalcImageRectangle(IMessageControl ctrl)
        {
            Rectangle r;
            Vector2 pos = Vector2.Zero;
            r.Width = this.currentInfo.ImageDisplaySizeWidth;
            r.Height = this.currentInfo.ImageDisplaySizeHeight;

            // figure out image position
            switch (this.currentInfo.ImagePosition)
            {
                case MessageInformation.ImageDisplayPosition.UpperLeft:
                    pos = ctrl.PointToScreen(0, 0);
                    pos += new Vector2(this.leftMargin, this.topMargin);
                    break;

                case MessageInformation.ImageDisplayPosition.UpperRight:
                    pos = ctrl.PointToScreen(ctrl.Bounds.Width - r.Width, 0);
                    pos += new Vector2(-this.leftMargin, this.topMargin);
                    break;

                case MessageInformation.ImageDisplayPosition.LowerLeft:
                    pos = ctrl.PointToScreen(0, ctrl.Bounds.Height - r.Height);
                    pos += new Vector2(this.leftMargin, -this.topMargin);
                    break;

                case MessageInformation.ImageDisplayPosition.LowerRight:
                    pos = ctrl.PointToScreen(ctrl.Bounds.Width - r.Width, ctrl.Bounds.Height - r.Height);
                    pos -= new Vector2(this.leftMargin, this.topMargin);
                    break;
            }

            r.X = (int)pos.X;
            r.Y = (int)pos.Y;
            this.imageRect = r;
            return r;
        }

        private Rectangle CalcTextRectangle(IMessageControl ctrl)
        {
            Rectangle r = Rectangle.Empty;
            Vector2 pos = Vector2.Zero;

            if (this.currentInfo.ImageEffect != null)
            {
                // figure out text position
                switch (this.currentInfo.ImagePosition)
                {
                    case MessageInformation.ImageDisplayPosition.LowerLeft:
                    case MessageInformation.ImageDisplayPosition.UpperLeft:
                        pos = ctrl.PointToScreen(this.currentInfo.ImageDisplaySizeWidth, 0);
                        pos += new Vector2(this.leftMargin + this.imageTextMargin, this.topMargin);
                        break;

                    case MessageInformation.ImageDisplayPosition.LowerRight:
                    case MessageInformation.ImageDisplayPosition.UpperRight:
                        pos = ctrl.PointToScreen(0, 0);
                        pos += new Vector2(this.leftMargin, this.topMargin);
                        break;
                }
                r = new Rectangle((int)pos.X, (int)pos.Y, 0, 0);
                r.Width = (int)(ctrl.Width - (this.leftMargin + this.rightMargin + this.imageTextMargin + this.currentInfo.ImageDisplaySizeWidth));
            }
            else
            {
                pos = ctrl.PointToScreen(0, 0);
                pos += new Vector2(this.leftMargin, this.topMargin);
                r = new Rectangle((int)pos.X, (int)pos.Y, 0, 0);
                r.Width = ctrl.Width - (this.leftMargin + this.rightMargin);
            }

            r.Height = ctrl.Height - (this.topMargin + this.bottomMargin);
            this.textRect = r;
            return r;
        }

      
    }
}


