/*
Created by: X
http://www.createdbyx.com/

xMessages is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace xMessages
{
    public class MessageInformation : IDisposable,  ICloneable
    {

        public enum ImageDisplayPosition
        {
            UpperLeft,
            UpperRight,
            LowerLeft,
            LowerRight
        }

        public delegate void StartedEventHandler(MessageInformation e);
        public event StartedEventHandler Started;

        public delegate void EndedEventHandler(MessageInformation e);
        public event EndedEventHandler Ended;

        internal System.Collections.Specialized.StringCollection text;
        internal SpriteFont font;
        private int imageDisplaySizeWidth;
        private int imageDisplaySizeHeight;
        private TimeSpan displayTime = new TimeSpan(0, 0, 0);
        private bool disposeOnEnded = false;
        private ImageDisplayPosition imagePos = ImageDisplayPosition.UpperLeft;
        private ITextEffect textEffect = new TextEffects.PlainText();
        internal TimeSpan startTime = TimeSpan.Zero;
        private IImageEffect imageEffect = new ImageEffects.PlainImage();

        public T GetTextEffect<T>()
        {
            return (T)this.textEffect;
        }

        public T GetImageEffect<T>()
        {
            return (T)this.imageEffect;
        }
        
        public IImageEffect ImageEffect
        {
            get { return imageEffect; }
            set { imageEffect = value; }
        }
	

        public TimeSpan StartTime
        {
            get { return this.startTime; }
        }

        internal void SetStartTime(TimeSpan gameTime)
        {
            this.startTime = gameTime;
        }

        public ITextEffect TextEffect
        {
            get { return textEffect; }
            set {
                this.textEffect = null;
                this.textEffect = value;
            }
        }

        /// <summary>
        /// Gets/Sets the position that the image will be located at.
        /// </summary>
        public ImageDisplayPosition ImagePosition
        {
            get { return imagePos; }
            set { imagePos = value; }
        }

        /// <summary>
        /// If true the Dispose method will be called automatically when the message has ended.
        /// </summary>
        public bool DisposeOnEnded
        {
            get { return disposeOnEnded; }
            set { disposeOnEnded = value; }
        }

        internal protected void DoStarted()
        {
            if (this.Started != null) { this.Started(this); }
        }

        internal protected void DoEnded()
        {
            if (this.Ended != null) { this.Ended(this); }
        }


        public MessageInformation()
            : base()
        {

        }

        public MessageInformation(System.Collections.Specialized.StringCollection txt, SpriteFont fnt, int imageWidth, int imageHeight, ITextEffect textFX, TimeSpan t)
        {
            this.Text = txt;
            this.font = fnt;
            this.ImageDisplaySizeWidth = imageWidth;
            this.imageDisplaySizeHeight = imageHeight;
            this.textEffect = textFX;
            this.displayTime = t;
        }

        public MessageInformation(System.Collections.Specialized.StringCollection txt, SpriteFont fnt, ITextEffect textFX, TimeSpan t)
        {
            this.Text = txt;
            this.font = fnt;
            this.textEffect = textFX;
            this.displayTime = t;
        }

        public MessageInformation(System.Collections.Specialized.StringCollection txt, SpriteFont fnt, int imageWidth, int imageHeight, TimeSpan t)
        {
            this.Text = txt;
            this.font = fnt;
             this.ImageDisplaySizeWidth = imageWidth;
            this.imageDisplaySizeHeight = imageHeight;
            this.displayTime = t;
        }

        public MessageInformation(System.Collections.Specialized.StringCollection txt, SpriteFont fnt, TimeSpan t)
        {
            this.Text = txt;
            this.font = fnt;
            this.displayTime = t;
        }

        /// <summary>
        /// Gets/Sets the total display time for the message.
        /// </summary>
        public TimeSpan DisplayTime
        {
            get { return displayTime; }
            set { displayTime = value; }
        }



        /// <summary>
        /// Gets/Sets the width of the displayed image.
        /// </summary>
        public int ImageDisplaySizeWidth
        {
            get { return imageDisplaySizeWidth; }
            set { imageDisplaySizeWidth = value; }
        }

        /// <summary>
        /// Gets/Sets the height of the displayed image.
        /// </summary>
        public int ImageDisplaySizeHeight
        {
            get { return imageDisplaySizeHeight; }
            set { imageDisplaySizeHeight = value; }
        }



        /// <summary>
        /// Gets/Sets the font to use when displaying the message.
        /// </summary>
        public SpriteFont Font
        {
            get { return font; }
            set { font = value; }
        }

        /// <summary>
        /// Gets/Sets the text that will be displayed in the message.
        /// </summary>
        /// <remarks>Instead of using \n in your strings add each sentance or paragraph of text indivdually in the collection. 
        /// To place a empty line simple ass a empty string.</remarks>
        public System.Collections.Specialized.StringCollection Text
        {
            get { return text; }
            set
            {
                text = value;
                int idx = 0;
                while (idx < this.text.Count)
                {
                    string[] tmpitems = this.text[idx].Split(new string[] { "\n" }, StringSplitOptions.None);
                    if (tmpitems.Length > 1)
                    {
                        this.text.RemoveAt(idx);
                        for (int idy = 0; idy < tmpitems.Length; idy++)
                        {
                            this.text.Insert(idx, tmpitems[idy]);
                            idx++;
                        }
                    }
                    else
                    {
                        idx++;
                    }
                }
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            this.font = null;
        }

        #endregion

              /// <summary>
        /// Updates any message animations.
        /// </summary>
        /// <param name="gameTime">Current snapshot of the game time.</param>
        internal  void Update(GameTime gameTime, Rectangle textRect, Rectangle imageRect)
        {
            if (this.textEffect != null)
            {
                this.textEffect.Update(gameTime, this, textRect);
            }
            if (this.imageEffect != null)
            {
                this.imageEffect.Update(gameTime, this, imageRect);
            }
        }
        #region ICloneable Members

        /// <summary>
        /// Creates a duplicate MessageInformation object.
        /// </summary>
        public object Clone()
        {
            MessageInformation msg = new MessageInformation();
            msg.displayTime = this.displayTime;
            msg.disposeOnEnded = this.disposeOnEnded;
            msg.font = this.font;
            msg.imageDisplaySizeHeight = this.imageDisplaySizeHeight;
            msg.imageDisplaySizeWidth = this.imageDisplaySizeWidth;
            msg.imagePos = this.imagePos;
            msg.startTime = this.startTime;
            msg.text = this.text;
            msg.textEffect = this.textEffect;
            msg.imageEffect = this.imageEffect;

            return msg;
        }

        #endregion
    }
}
