/*
Created by: X
http://www.createdbyx.com/

xMessages is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xMessages.TextEffects
{
    public class TypedText : ITextEffect, IDisposable
    {
        protected int currentLine = -1;
        protected int currentPos = -1;
        private TimeSpan textSpeed = TimeSpan.Zero;
        private StringCollection textToDraw = new StringCollection();
        private TimeSpan nextUpdate;
        private Color textColor = Color.White;
        private string nextWord = string.Empty;
        private bool finishedDrawing = false;
        private Rectangle prevRect;


        /// <summary>
        /// Gets/Sets the color that the message will be drawn as.
        /// </summary>
        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }

        /// <summary>
        /// Gets/Sets the time delay between displaying the message.
        /// </summary>
        public TimeSpan TextSpeed
        {
            get { return textSpeed; }
            set { textSpeed = value; }
        }

        private string GetNextWord(MessageInformation e, int index, int pos)
        {
            string line = string.Empty;
            if (index < e.text.Count) line = e.text[index] + " ";
            if (line.Trim() == string.Empty) return string.Empty;
            string completeLine = line.Substring(0, pos + 1);
            int nextspacepos = line.IndexOf(" ", pos + 1);
            if (nextspacepos != -1)
            {
                completeLine += line.Substring(pos + 1, nextspacepos - (pos + 1));
            }

            int lastspace = completeLine.LastIndexOf(" ");
            if (lastspace != -1)
            {
                completeLine = completeLine.Substring(lastspace + 1);
            }

            return completeLine + " ";
        }

        #region ITextEffect Members

        /// <summary>
        /// Resets the message.
        /// </summary>
        public void Reset()
        {
            this.currentLine = -1;
            this.currentPos = -1;
            this.nextWord = string.Empty;
            this.finishedDrawing = false;
            this.nextUpdate = TimeSpan.Zero;
            this.textToDraw.Clear();
        }

        public void Update(Microsoft.Xna.Framework.GameTime gameTime, MessageInformation e, Rectangle rect)
        {
            if (this.nextUpdate == TimeSpan.Zero)
            {
                this.nextUpdate = gameTime.TotalGameTime;
                this.prevRect = rect;
            }

            // check if it is time to update
            if (gameTime.TotalGameTime >= this.nextUpdate)
            {
                if (e.font != null)
                {
                    // detect a change in the rect size
                    if (this.prevRect.Width != rect.Width | this.prevRect.Height != rect.Height)
                    {
                        int tmpline = this.currentLine;
                        int tmpPos = this.currentPos;
                        this.currentLine = -1;
                        this.currentPos = -1;
                        this.textToDraw.Clear();
                        this.nextWord = string.Empty;
                        this.finishedDrawing = false;
                        while ((this.currentPos != tmpPos) || (this.currentLine != tmpline))
                        {
                            BuildText(e, rect);
                        }
                        this.prevRect = rect;
                    }
                    else
                    {
                        // continue building text normally if not finished
                        if (!this.finishedDrawing) BuildText(e, rect);
                    }

                    // set the time for the next update
                    this.nextUpdate += this.textSpeed;
                }
            }
        }

        private void BuildText(MessageInformation e, Rectangle rect)
        {
            if (this.currentLine == -1) this.currentLine++;
            this.currentPos++;
            if (e.text.Count > 0 && this.currentPos > e.text[this.currentLine].Length)
            {
                this.currentLine++;
                if (this.currentLine > e.text.Count - 1)
                {
                    this.currentLine = e.text.Count - 1;
                    this.currentPos = e.text[this.currentLine].Length;
                    this.finishedDrawing = true;
                }
                else
                {
                    this.currentPos = 0;
                    this.textToDraw.Add(string.Empty);
                }
            }

            string line = string.Empty;

            // check if there is any text already in the collection
            if (this.textToDraw.Count == 0)
            {
                if (this.nextWord != string.Empty)
                {
                    // add first character in first word to the collection
                    this.textToDraw.Add(this.nextWord[0].ToString());
                    line = this.nextWord[0].ToString();
                    // remove first character from word
                    this.nextWord = this.nextWord.Remove(0, 1);
                }
            }
            else
            {
                if (this.nextWord != string.Empty)
                {
                    // get last line of text to draw
                    line = this.textToDraw[this.textToDraw.Count - 1];
                    // prepend the first character to the line
                    line += this.nextWord[0];
                    // remove first character from word
                    this.nextWord = this.nextWord.Remove(0, 1);
                    // update the line value
                    this.textToDraw[this.textToDraw.Count - 1] = line;
                }

            }

            // check to see if we need to get the next word to draw
            if (this.nextWord == string.Empty)
            {
                // get the next word
                this.nextWord = this.GetNextWord(e, this.currentLine, this.currentPos);
                Vector2 size = e.font.MeasureString(line + nextWord.Trim());
                // check if the line plus this next word will fit inside the rectangle
                if (size.X > rect.Width)
                {
                    // if it does not fir start a new line of text
                    this.textToDraw.Add(string.Empty);
                }
            }

        }

        public void Draw(Microsoft.Xna.Framework.GameTime gameTime, MessageInformation e, Rectangle rect, SpriteBatch spriteBatch)
        {
            if (e == null) return;
            if (spriteBatch == null) return;
            General.DrawString(spriteBatch, e.Font, this.textToDraw, rect, this.textColor, false);
        }

        #endregion



        #region IDisposable Members

        public void Dispose()
        {
            this.textToDraw.Clear();
            this.textToDraw = null;
        }

        #endregion
    }
}
